@@ -9,9 +9,16 @@ Author: RhythmLian (https://rhythmlian.cn)
 """
 from . import size_format, get_fileinfo
 from concurrent.futures import ThreadPoolExecutor, wait
-from ..ThreadTools import FileWriters
 from ..SystemTools import get_core_num
-from .. import user_lang, qs_default_console, qs_error_string, qs_info_string, qs_warning_string, headers
+from .. import (
+    user_lang,
+    qs_default_console,
+    qs_error_string,
+    qs_info_string,
+    qs_warning_string,
+    headers,
+    requirePackage,
+)
 from threading import Lock
 from requests import get
 import signal
@@ -45,20 +52,35 @@ def GetBlockSize(sz):
 def ask_overwrite(path: str):
     from ..__config__ import prompt
 
-    return prompt({
-        'type': 'confirm',
-        'name': 'overwrite',
-        'message': f'"{path}" 已存在, 是否覆盖?' if user_lang == 'zh' else f'"{path}" already exists, overwrite?',
-        'default': False
-    })['overwrite']
+    return prompt(
+        {
+            "type": "confirm",
+            "name": "overwrite",
+            "message": f'"{path}" 已存在, 是否覆盖?'
+            if user_lang == "zh"
+            else f'"{path}" already exists, overwrite?',
+            "default": False,
+        }
+    )["overwrite"]
 
 
 class Downloader:
     from ..TuiTools.Bar import DataTransformBar
 
-    def __init__(self, url: str, num: int, name: str = '', proxy: str = '',
-                 referer: str = '', output_error: bool = False, failed2exit: bool = False, exit_if_exist: bool = False,
-                 disableStatus: bool = False, disableParallel: bool = False):
+    def __init__(
+        self,
+        url: str,
+        num: int,
+        name: str = "",
+        proxy: str = "",
+        referer: str = "",
+        output_error: bool = False,
+        failed2exit: bool = False,
+        exit_if_exist: bool = False,
+        disableStatus: bool = False,
+        disableParallel: bool = False,
+        write_to_memory: bool = False,
+    ):
         """
         qs普通文件下载引擎
 
@@ -81,21 +103,27 @@ class Downloader:
         self.exit_if_exist = exit_if_exist
         self.enabled = True
         self.disableStatus = disableStatus
+        self.write_to_memory = write_to_memory
         if not self.disableStatus:
-            with qs_default_console.status('Getting file info..' if user_lang != 'zh' else '获取文件信息中..'):
+            with qs_default_console.status(
+                "Getting file info.." if user_lang != "zh" else "获取文件信息中.."
+            ):
                 self.url, self.name, r = get_fileinfo(url, proxy, referer)
         else:
             self.url, self.name, r = get_fileinfo(url, proxy, referer)
         if not (self.url and self.name and r):
             info_flag = False
             if not self.disableStatus:
-                qs_default_console.print(qs_error_string if failed2exit else qs_warning_string,
-                                         'Get File information failed, please check network!'
-                                         if user_lang != 'zh' else '获取文件信息失败，请检查网络!')
+                qs_default_console.print(
+                    qs_error_string if failed2exit else qs_warning_string,
+                    "Get File information failed, please check network!"
+                    if user_lang != "zh"
+                    else "获取文件信息失败，请检查网络!",
+                )
             if failed2exit:
                 self.enabled = False
                 return
-        if self.name and '.' not in self.name:
+        if self.name and "." not in self.name:
             self.name = os.path.basename(url)
         if name:
             self.name = name
@@ -105,50 +133,68 @@ class Downloader:
             return
 
         if proxy:
-            self.proxies = {
-                'http': 'http://'+proxy,
-                'https': 'https://'+proxy
-            }
+            self.proxies = {"http": "http://" + proxy, "https": "https://" + proxy}
         self.headers = headers
         if referer:
-            self.headers['Referer'] = referer
+            self.headers["Referer"] = referer
         if not self.url:
             qs_default_console.print(qs_error_string, self.name)
-            raise Exception('Connection Error!' if user_lang != 'zh' else '连接失败!')
+            raise Exception("Connection Error!" if user_lang != "zh" else "连接失败!")
         try:
             if not info_flag:
                 raise KeyError
-            self.size = int(r.headers['content-length'])
+            self.size = int(r.headers["content-length"])
             self.main_progress = Downloader.DataTransformBar()
-            self.dl_id = self.main_progress.add_task('Download', filename=self.name, start=False)
+            self.dl_id = self.main_progress.add_task(
+                "Download", filename=self.name, start=False
+            )
             self.main_progress.update(self.dl_id, total=self.size)
             if self.size < 5e6 or disableParallel:
                 if not self.disableStatus:
-                    qs_default_console.print(qs_info_string, 'FILE SIZE' if user_lang != 'zh' else '文件大小'
-                                             , size_format(self.size))
+                    qs_default_console.print(
+                        qs_info_string,
+                        "FILE SIZE" if user_lang != "zh" else "文件大小",
+                        size_format(self.size),
+                    )
                 self.size = -self.size
             else:
                 self.fileBlock = GetBlockSize(self.size)
         except KeyError:
             self.size = -1
             self.main_progress = Downloader.DataTransformBar(False)
-            self.dl_id = self.main_progress.add_task('Download', filename=self.name, start=False)
+            self.dl_id = self.main_progress.add_task(
+                "Download", filename=self.name, start=False
+            )
         if self.size > 0:
             self.pool = ThreadPoolExecutor(max_workers=self.num)
             self.futures, self.fileLock = [], Lock()
             self.job_queue = queue.Queue()
-            if os.path.exists(self.name + '.qs_dl'):
-                self.ctn_file = open(self.name + '.qs_dl', 'r+')
-                self.ctn = [int(i) for i in self.ctn_file.read().strip().split()]
+            if os.path.exists(self.name + ".qs_dl"):
+                self.ctn_file = open(self.name + ".qs_dl", "r+")
+                self.ctn = {int(i) for i in self.ctn_file.read().strip().split()}
+            elif not self.write_to_memory:
+                self.ctn_file = open(self.name + ".qs_dl", "w")
+                self.ctn = set()
             else:
-                self.ctn_file = open(self.name + '.qs_dl', 'w')
-                self.ctn = []
-            self.writers = FileWriters(self.name, max(2, int(core_num / 2)), "rb+" if self.ctn else "wb")
+                self.ctn = set()
+            self.writers = (
+                requirePackage(".ThreadTools", "FileWriters")(
+                    self.name, max(2, int(core_num / 2)), "rb+" if self.ctn else "wb"
+                )
+                if not self.write_to_memory
+                else requirePackage(".ThreadTools", "MemWriter")()
+            )
             if not self.disableStatus:
-                qs_default_console.print(qs_info_string, 'FILE  SIZE' if user_lang != 'zh' else '文件大小'
-                                         , size_format(self.size, align=True))
-                qs_default_console.print(qs_info_string, 'THRAED NUM' if user_lang != 'zh' else '线程数量'
-                                         , '%7d' % num)
+                qs_default_console.print(
+                    qs_info_string,
+                    "FILE  SIZE" if user_lang != "zh" else "文件大小",
+                    size_format(self.size, align=True),
+                )
+                qs_default_console.print(
+                    qs_info_string,
+                    "THRAED NUM" if user_lang != "zh" else "线程数量",
+                    "%7d" % num,
+                )
 
     def _kill_self(self, signum, frame):
         """
@@ -163,12 +209,17 @@ class Downloader:
         if self.size > 0:
             self.main_progress.stop_task(self.dl_id)
             self.main_progress.stop()
-            qs_default_console.print(qs_info_string,
-                                     'Get Ctrl-C, exiting...' if user_lang != 'zh' else '捕获Ctrl-C, 正在退出...')
-            self.ctn_file.close()
+            qs_default_console.print(
+                qs_info_string,
+                "Get Ctrl-C, exiting..." if user_lang != "zh" else "捕获Ctrl-C, 正在退出...",
+            )
+            if not self.write_to_memory:
+                self.ctn_file.close()
             self.pool.shutdown(wait=False)
             self.writers.wait()
-            qs_default_console.print(qs_info_string, 'Deal Done!' if user_lang != 'zh' else '处理完成!')
+            qs_default_console.print(
+                qs_info_string, "Deal Done!" if user_lang != "zh" else "处理完成!"
+            )
         os._exit(0)
 
     def _dl(self, start):
@@ -183,9 +234,11 @@ class Downloader:
         try:
             _sz = min(start + self.fileBlock, self.size - 1)
             _headers = self.headers.copy()
-            _headers['Range'] = 'bytes={}-{}'.format(start, _sz)
-            _content = b''
-            for chunk in get(self.url, headers=_headers, timeout=50, proxies=self.proxies).iter_content(65536):
+            _headers["Range"] = "bytes={}-{}".format(start, _sz)
+            _content = b""
+            for chunk in get(
+                self.url, headers=_headers, timeout=50, proxies=self.proxies
+            ).iter_content(65536):
                 _content += chunk
                 if not self.disableStatus:
                     self.main_progress.advance(self.dl_id, sys.getsizeof(chunk))
@@ -195,10 +248,11 @@ class Downloader:
                 qs_default_console.print(qs_error_string, repr(e))
             self.job_queue.put(start)
         else:
-            self.fileLock.acquire()
-            self.ctn.append(start)
-            self.ctn_file.write('%d\n' % start)
-            self.fileLock.release()
+            self.ctn.add(start)
+            if not self.write_to_memory:
+                self.fileLock.acquire()
+                self.ctn_file.write("%d\n" % start)
+                self.fileLock.release()
 
     def _single_dl(self):
         """
@@ -216,9 +270,15 @@ class Downloader:
                 self.main_progress.start_task(self.dl_id)
             else:
                 self.main_progress.update(self.dl_id, total=-1)
-        with open(self.name, 'wb') as f:
+        if self.write_to_memory == False:
+            with open(self.name, "wb") as f:
+                for chunk in r.iter_content(32768):
+                    f.write(chunk)
+                    self.main_progress.advance(self.dl_id, sys.getsizeof(chunk))
+        else:
+            self.name = b""
             for chunk in r.iter_content(32768):
-                f.write(chunk)
+                self.name += chunk
                 self.main_progress.advance(self.dl_id, sys.getsizeof(chunk))
 
     def run(self):
@@ -240,15 +300,21 @@ class Downloader:
         if self.size > 0:
             if not self.disableStatus:
                 self.main_progress.start_task(self.dl_id)
-            if not self.ctn:
-                with open(self.name, "wb") as fp:
-                    fp.truncate(self.size)
-            if not self.disableStatus:
-                self.main_progress.advance(self.dl_id, self.fileBlock*len(self.ctn))
-            for i in range(0, self.size, self.fileBlock):
-                if self.ctn and i in self.ctn:
-                    continue
-                else:
+            if not self.write_to_memory:
+                if not self.ctn:
+                    with open(self.name, "wb") as fp:
+                        fp.truncate(self.size)
+                if not self.disableStatus:
+                    self.main_progress.advance(
+                        self.dl_id, self.fileBlock * len(self.ctn)
+                    )
+                for i in range(0, self.size, self.fileBlock):
+                    if self.ctn and i in self.ctn:
+                        continue
+                    else:
+                        self.job_queue.put(i)
+            else:
+                for i in range(0, self.size, self.fileBlock):
                     self.job_queue.put(i)
             retry_cnt = 0
             while not self.job_queue.empty():
@@ -259,24 +325,45 @@ class Downloader:
                         self.futures.append(self.pool.submit(self._dl, cur))
                 wait(self.futures)
                 if not self.job_queue.empty() and retry_cnt > 2:
-                    qs_default_console.print(qs_warning_string, 'Exists File Block Lost, Retrying after 0.5 sec'
-                                             if user_lang != 'zh' else '存在文件块丢失，0.5秒后重试')
+                    qs_default_console.print(
+                        qs_warning_string,
+                        "Exists File Block Lost, Retrying after 0.5 sec"
+                        if user_lang != "zh"
+                        else "存在文件块丢失，0.5秒后重试",
+                    )
                     time.sleep(0.5)
                 retry_cnt += 1
             self.writers.wait()
-            self.ctn_file.close()
-            os.remove(self.name + '.qs_dl')
+            if not self.write_to_memory:
+                self.ctn_file.close()
+                os.remove(self.name + ".qs_dl")
+            else:
+                self.name = self.writers.content
         else:
             self._single_dl()
         if not self.disableStatus:
             self.main_progress.stop()
-            qs_default_console.print(qs_info_string, self.name, 'download done!' if user_lang != 'zh' else '下载完成!')
+            qs_default_console.print(
+                qs_info_string,
+                self.name,
+                "download done!" if user_lang != "zh" else "下载完成!",
+            )
         return self.name
 
 
-def normal_dl(url, set_name: str = '', set_proxy: str = '', set_referer: str = '',
-              thread_num: int = min(16, core_num * 4), output_error: bool = False, failed2exit: bool = False, exit_if_exist: bool = False,
-              disableStatus: bool = False, disableParallel: bool = False):
+def normal_dl(
+    url,
+    set_name: str = "",
+    set_proxy: str = "",
+    set_referer: str = "",
+    thread_num: int = min(16, core_num * 4),
+    output_error: bool = False,
+    failed2exit: bool = False,
+    exit_if_exist: bool = False,
+    disableStatus: bool = False,
+    disableParallel: bool = False,
+    write_to_memory: bool = False,
+):
     """
     自动规划下载线程数量并开始并行下载
 
@@ -284,21 +371,30 @@ def normal_dl(url, set_name: str = '', set_proxy: str = '', set_referer: str = '
 
     :param disableStatus:
     :param url: 文件url
-    :param set_name: 设置文件名（默认采用url所指向的资源名）
+    :param set_name: 设置文件路径
     :param set_proxy: 设置代理（默认无代理）
     :param set_referer: 设置referer
     :param thread_num: 线程数
     :param output_error: 输出报错信息
-    :param failed2exit: 获取文件信息失败则不下载，否则qs将继续尝试下载
+    :param failed2exit: 获取文件信息失败则不下载, 否则qs将继续尝试下载
     :param exit_if_exist: 如果文件已存在则不下载
     :param disableStatus: 是否显示当前任务状态
     :param disableParallel: 是否禁用并行下载
-    :return: file name
+    :param write_to_memory: 是否将下载内容直接写入内存（默认写入文件）
+    :return: 文件路径或二进制内容 | file path or bytes
     """
     if set_name and os.path.exists(set_name) and not ask_overwrite(set_name):
-        return ''
+        return ""
     return Downloader(
-        url=url, num=thread_num, name=set_name, proxy=set_proxy,
-        referer=set_referer, output_error=output_error, failed2exit=failed2exit, exit_if_exist=exit_if_exist,
-        disableStatus=disableStatus, disableParallel=disableParallel
+        url=url,
+        num=thread_num,
+        name=set_name,
+        proxy=set_proxy,
+        referer=set_referer,
+        output_error=output_error,
+        failed2exit=failed2exit,
+        exit_if_exist=exit_if_exist,
+        disableStatus=disableStatus,
+        disableParallel=disableParallel,
+        write_to_memory=write_to_memory,
     ).run()
