@@ -5,6 +5,7 @@ class QsCache:
     """
     qs存储在~/.qs_cache的缓存
     """
+
     import os
     import pickle
 
@@ -16,14 +17,13 @@ class QsCache:
     def get(self, key: str):
         if not QsCache.os.path.exists(self.path + key):
             return None
-        with open(self.path + key, 'rb') as f:
+        with open(self.path + key, "rb") as f:
             return QsCache.pickle.loads(f.read())
 
     def set(self, key: str, value):
-        with open(self.path + key, 'wb') as f:
+        with open(self.path + key, "wb") as f:
             QsCache.pickle.dump(value, f)
 
     def delete(self, key: str):
         if QsCache.os.path.exists(self.path + key):
             QsCache.os.remove(self.path + key)
-
